#import "BarcodeOptionViewController.h"
#import "BarcodeOption_VC.h"
#import "SystemSetting.h"
@interface BarcodeOptionViewController ()
@property (weak, nonatomic) IBOutlet UITableView *tableView;
@property (strong, nonatomic) NSArray *barcodeParamData;
@end

@implementation BarcodeOptionViewController{
    SystemSetting * systemSetting;
}

- (void)viewDidLoad {
    [super viewDidLoad];
    [AppDelegate showGlobalProgressHUDWithTitle:@"Loading..."];
    appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
    [appDelegate.mReader setDelegate:self];
    
    systemSetting = [SystemSetting sharedSystemSetting];
    if (appDelegate.mReader == nil) {
        return;
    }else{
        if (!appDelegate.mReader.isRFIDModule) {
            UIBarButtonItem * rightItem =[[UIBarButtonItem alloc]initWithTitle:@"Setting" style:UIBarButtonItemStyleDone target:self action:@selector(setting)];
            self.navigationItem.rightBarButtonItem = rightItem;
        }
    }
}
- (void)setting{
    BarcodeOption_VC * barcodeViewController=  (BarcodeOption_VC *)[self.storyboard instantiateViewControllerWithIdentifier:@"BarcodeOption_VC"];
    [self.navigationController pushViewController:barcodeViewController animated:YES];
}
- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
    NSNotificationCenter *center = [NSNotificationCenter defaultCenter];
    [center addObserver:self selector:@selector(AsReaderGUNDisconnected:) name:@"AsReaderGUNDisconnected" object:nil];
    [appDelegate.mReader setDelegate:self];
    [appDelegate.mReader setScanMode:BarcodeScanMode];
}
-(void)viewDidAppear:(BOOL)animated{
    [appDelegate.mReader setPropBarcodeMode:YES];
    NSArray *paramData = [[NSArray alloc] initWithObjects:
                          [NSNumber numberWithInt:UPCA],
                          [NSNumber numberWithInt:UPCE0],
                          [NSNumber numberWithInt:UPCE1],
                          [NSNumber numberWithInt:EAN8],
                          [NSNumber numberWithInt:EAN13],
                          [NSNumber numberWithInt:Code128],
                          [NSNumber numberWithInt:Code39],
                          [NSNumber numberWithInt:Code39Pharmaceutical],
                          [NSNumber numberWithInt:Code93],
                          [NSNumber numberWithInt:Code11],
                          [NSNumber numberWithInt:I2of5],
                          [NSNumber numberWithInt:Codabar],
                          [NSNumber numberWithInt:MSI],
                          [NSNumber numberWithInt:X2of5],
                          [NSNumber numberWithInt:PDF417],
                          [NSNumber numberWithInt:MacroPDF417],
                          [NSNumber numberWithInt:MicroPDF],
                          [NSNumber numberWithInt:Matrix],
                          [NSNumber numberWithInt:MaxiCode],
                          [NSNumber numberWithInt:QRCode],
                          [NSNumber numberWithInt:AztecCode],
                          [NSNumber numberWithInt:NEC2of5],
                          [NSNumber numberWithInt:R2of5],
                          [NSNumber numberWithInt:A2of5],
                          [NSNumber numberWithInt:GS1128],
                          [NSNumber numberWithInt:Telepen],
                          [NSNumber numberWithInt:RSS14],
                          [NSNumber numberWithInt:RSSLimit],
                          [NSNumber numberWithInt:RSSExp],
                          [NSNumber numberWithInt:TriopticCode],
                          [NSNumber numberWithInt:CodablockA],
                          [NSNumber numberWithInt:CodablockF],
                          [NSNumber numberWithInt:ComCode],
                          [NSNumber numberWithInt:TLC39],
                          [NSNumber numberWithInt:ChinaPost],
                          [NSNumber numberWithInt:KoreaPost],
                          [NSNumber numberWithInt:HanXinCode],
                          nil];
    NSMutableArray *arrParam = [[NSMutableArray alloc] init];
    for (NSNumber *paramName in paramData) {
        AsParamValue * val = [appDelegate.mReader getBarcodeParam:paramName];
        if(val != nil)
        {
            [arrParam addObject:val];
        }
    }
    if ([systemSetting getBarcodeParamData] && [[systemSetting getBarcodeParamData] count] > 0 ) {
        self.barcodeParamData = (NSArray *)[systemSetting getBarcodeParamData];
    }else{
        self.barcodeParamData = (NSArray *)arrParam;
    }
    [self.tableView reloadData];
    [AppDelegate dismissGlobalHUD];
}

- (void)viewWillDisappear:(BOOL)animated {
    [super viewWillDisappear:animated];
    [[NSNotificationCenter defaultCenter] removeObserver:self name:@"AsReaderGUNDisconnected" object:nil];
    [appDelegate.mReader setScanMode:RFIDScanMode];
    [appDelegate.mReader setBarcodeMode:NO isKeyAction:NO];
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
}

#pragma mark - UITableViewDataSource
- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return self.barcodeParamData.count;
}

#pragma mark - UITableViewDelegate
- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:@"BarcodeCell" forIndexPath:indexPath];
    
    AsParamValue *paramValue = [self.barcodeParamData objectAtIndex:indexPath.row];
//    cell.textLabel.text = [AsParamName msg:paramValue.paramName];
    cell.textLabel.text = [AsParamName getName:paramValue.paramName];

    if (paramValue.value) {
        cell.accessoryType = UITableViewCellAccessoryCheckmark;
    } else {
        cell.accessoryType = UITableViewCellAccessoryNone;
    }
    
    return cell;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    UITableViewCell *cell = [tableView cellForRowAtIndexPath:indexPath];
    AsParamValue *paramValue = [self.barcodeParamData objectAtIndex:indexPath.row];
    
    if (cell.accessoryType == UITableViewCellAccessoryCheckmark) {
        cell.accessoryType = UITableViewCellAccessoryNone;
        paramValue.value = 0;
    } else {
        cell.accessoryType = UITableViewCellAccessoryCheckmark;
        paramValue.value = 1;
    }
}

- (IBAction)defaultAllSymBtnTapped:(id)sender {
    for (AsParamValue *paramValue in self.barcodeParamData) {
        switch(paramValue.paramName) {
            case UPCA:
                [paramValue setEnabled:YES];
                break;
            case UPCE0:
                [paramValue setEnabled:YES];
                break;
            case UPCE1:
                [paramValue setEnabled:NO];
                break;
            case EAN8:
                [paramValue setEnabled:YES];
                break;
            case EAN13:
                [paramValue setEnabled:YES];
                break;
            case Code128:
                [paramValue setEnabled:YES];
                break;
            case Code39:
                [paramValue setEnabled:YES];
                break;
            case Code39Pharmaceutical:
                [paramValue setEnabled:NO];
                break;
            case Code93:
                [paramValue setEnabled:YES];
                break;
            case Code11:
                [paramValue setEnabled:NO];
                break;
            case I2of5:
                [paramValue setEnabled:YES];
                break;
            case Codabar:
                [paramValue setEnabled:YES];
                break;
            case MSI:
                [paramValue setEnabled:NO];
                break;
            case X2of5:
                [paramValue setEnabled:NO];
                break;
            case PDF417:
                [paramValue setEnabled:YES];
                break;
            case MacroPDF417:
                [paramValue setEnabled:YES];
                break;
            case MicroPDF:
                [paramValue setEnabled:NO];
                break;
            case Matrix:
                [paramValue setEnabled:NO];
                break;
            case MaxiCode:
                [paramValue setEnabled:YES];
                break;
            case QRCode:
                [paramValue setEnabled:YES];
                break;
            case AztecCode:
                [paramValue setEnabled:YES];
                break;
            case NEC2of5:
                [paramValue setEnabled:YES];
                break;
            case R2of5:
                [paramValue setEnabled:NO];
                break;
            case A2of5:
                [paramValue setEnabled:NO];
                break;
            case GS1128:
                [paramValue setEnabled:YES];
                break;
            case Telepen:
                [paramValue setEnabled:NO];
                break;
            case RSS14:
                [paramValue setEnabled:YES];
                break;
            case RSSLimit:
                [paramValue setEnabled:NO];
                break;
            case RSSExp:
                [paramValue setEnabled:NO];
                break;
            case TriopticCode:
                [paramValue setEnabled:NO];
                break;
            case CodablockA:
                [paramValue setEnabled:NO];
                break;
            case CodablockF:
                [paramValue setEnabled:NO];
                break;
            case ComCode:
                [paramValue setEnabled:YES];
                break;
            case TLC39:
                [paramValue setEnabled:NO];
                break;
            case ChinaPost:
                [paramValue setEnabled:NO];
                break;
            case KoreaPost:
                [paramValue setEnabled:NO];
                break;
            case HanXinCode:
                [paramValue setEnabled:NO];
                break;
            default:
                break;
        }
        
    }
    [self.tableView reloadData];
}

- (IBAction)disableAllSymBtnTapped:(id)sender {
    for (AsParamValue *paramValue in self.barcodeParamData) {
        [paramValue setEnabled:NO];
    }
    [self.tableView reloadData];
}

- (IBAction)enableAllSymBtnTapped:(id)sender {
    for (AsParamValue *paramValue in self.barcodeParamData) {
        [paramValue setEnabled:YES];
    }
    [self.tableView reloadData];
}

- (IBAction)saveSymBtnTapped:(id)sender {
//    ResultCode res = [appDelegate.mReader setBarcodeParam:self.barcodeParamData];
    ResultCode res = ResultNoError;
    [AppDelegate showGlobalProgressHUDWithTitle:@"Saving..."];

    for (AsParamValue *paramName in self.barcodeParamData) {
        res = [appDelegate.mReader setBarcodeParam:paramName];
        if (res != ResultNoError) {
            [AppDelegate dismissGlobalHUD];
            return;
        }
    }
    [AppDelegate dismissGlobalHUD];
    if (res == ResultNoError) {
        UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"Success" message:@"Saved successfully" delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil];
        [alert show];
    } else {
        UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"Error" message:@"Failed to save" delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil];
        [alert show];
    }
    [systemSetting setBarcodeParamData:self.barcodeParamData];

}

#pragma mark - Reader Event Handler Methods
- (BOOL)onAsReaderTriggerKeyEvent:(BOOL)status{
//    [appDelegate.mReader setSacnModel:BarcodeScanModel];
    return NO;
}
- (BOOL)onAsReaderLeftModeKeyEvent:(BOOL)status{
    if(appDelegate.mReader.isRFIDModule&&appDelegate.mReader.isBarcodeModule){
        return YES;
    }
    return NO;
}
- (BOOL)onAsReaderRightModeKeyEvent:(BOOL)status{
    if(appDelegate.mReader.isRFIDModule&&appDelegate.mReader.isBarcodeModule){
        return YES;
    }
    return NO;
}
- (void)updateDeviceState:(ResultCode)error {
    if (error != ResultNoError) {
        UIAlertView *alert = [[UIAlertView alloc]initWithTitle:@"Error" message:@"Failed to get response. Try again." delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil];
        [alert show];
        
        [self.navigationController popViewControllerAnimated:YES];
    }
}

- (void)readTag:(NSString *)tag rssi:(float)rssi phase:(float)phase {
    
}

- (void)changedActionState:(CommandType)action resultCode:(NSInteger)resultCode {
    
}
 
- (void)accessResult:(ResultCode)error actionState:(CommandType)action epc:(NSString *)epc data:(NSString *)data rssi:(float)rssi phase:(float)phase frequency:(float)frequency {
}

#pragma mark - NSNotification
- (void)AsReaderGUNDisconnected:(NSNotification *)notification {
    dispatch_async(dispatch_get_main_queue(), ^{
        appDelegate.mReader = nil;
        [AppDelegate dismissGlobalHUD];
        [self.navigationController popToRootViewControllerAnimated:YES];
    });
}

@end
